
/*----------------------------------------------------------------
   Copyright (c) Quinn-Curtis, 1993

   Filename:  ASYNC2.C
   Revision:  8.03  LAB
   Date:      1/18/93

   Description: -Routines for asynchronous communications
   Functions:
	 setxchars      sethandshakemode   sethardhandshake
	 setbuffervars  sendcom            opencom
	 closecom       checkcom           writecom
	 writelncom     readlncom		   setcombase
	 resetbuffer	setserial		   checkhandshake

-------------------------------------------------------------------*/



#ifdef __cplusplus
extern "C" {
#endif



#include <dos.h>
#include <bios.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef COM_1
#define COM_1       0  /* Com port 1 */
#define COM_2       1  /* Com port 2 */
#define ERR_NONE    0  /* No errors */
#define ERR_COM     1  /* Com port number error */
#define	ERR_BAUD    2  /* baud rate error */
#define	ERR_PARITY  3  /* parity error */
#define	ERR_STOP    4  /* stop bits error */
#define	ERR_WORD    5  /* world length error */
#define ERR_CHAR	6  /* no characters available */
#define	ERR_BUFF	7  /* buffer overflow */
#define ERR_TIME    8  /* time out */
#define ERR_MAXSTR  9  /* max string length */
#define ERR_INIT    10  /* com port not initialized */
#define HND_NONE	0  /* No handshaking */
#define	HND_HARD	1  /* hardware handshake */
#define	HND_SOFT	2  /* software handshake */
#define PAR_NONE	0  /* No parity*/
#define	PAR_ODD	    1  /* odd parity*/
#define	PAR_EVEN	3  /* even parity*/
#endif


typedef struct comportrec
{	char 	*combuffer;
#ifdef __cplusplus
	void interrupt (*oldfunc)(...);
#else
	void interrupt far (*oldfunc)(void);
#endif
	int bufferin;
	int bufferout;
	int bufferlength;
	int maxbuffer;
	int nearfull;
	int nearempty;
	int hardhandshake;
	int handshakemode;
	int combase;
	int	comintr;
	int thr;
	int	rbr;
	int	ier;
	int	lcr;
	int	mcr;
	int	lsr;
	int	msr;
	int	xerr;     /*async com board registers*/
	int	hardhsflag;
	int	xhsflag;
	int	comflag;
	int	putinbuffer;
	int imr;
	char xonchar;
	char xoffchar;
	char curchar;
} comportrec;


comportrec  cpvars[2] =
    {{0,0,0,0,0,1000,900,100,0x30,1,0x3f8,4,0,0,0,0,0,0,0,0,0,0,0,0,0,17,19,0},
     {0,0,0,0,0,1000,900,100,0x30,1,0x2f8,3,0,0,0,0,0,0,0,0,0,0,0,0,0,17,19,0}};

long timeout = 50000;

char getpntrelement(char *p, unsigned int n)
{
  return(*(p+n));
}

void putpntrelement(char *p, unsigned int n, char value)
{
  *(p+n) = value;
}

void setcombase(int cport, int cbase, int cintr)
{
  cpvars[cport].combase = cbase;
  cpvars[cport].comintr = cintr;
}

void sethandshakemode(int cport, int hsmode)
{
  cpvars[cport].handshakemode = hsmode;   /* 0 = no handshake */
                                           /* 1 = hardware handshake */
                                           /* 2 = xon xoff handshake*/
}

void sethardhandshake(int cport, int hs)
{
                       /*           0x0 no handshake            */
                       /*          0x10 cts handshaking         */
                       /*          0x20 dsr handshaking         */
					   /*          0x30 cts and dsr handshaking */
  cpvars[cport].hardhandshake = hs;
}
/*--------------------------------------------------------------*/

void setxchars(int cport, char xon, char xoff)
{
  cpvars[cport].xonchar = xon;
  cpvars[cport].xoffchar = xoff;
}
/*--------------------------------------------------------------*/

void clear_buffer(int cport)
{
  cpvars[cport].bufferin = 0;
  cpvars[cport].bufferout = 0;
}
/*--------------------------------------------------------------*/

void resetbuffer(int cport)
{
  cpvars[cport].bufferin = 0;
  cpvars[cport].bufferout = 0;
  cpvars[cport].bufferlength = 0;
}
/*--------------------------------------------------------------*/


void setbuffervars(int cport,int maxbuf,int nfull, int nempty)
{
  cpvars[cport].maxbuffer = maxbuf;
  cpvars[cport].nearfull = nfull;
  cpvars[cport].nearempty = nempty;
  cpvars[cport].comflag = 0;
}
/*--------------------------------------------------------------*/

void setserial(int cport,int baudrate)
{
	int  setdlab     =    0x80;          /*sets the divisor latch access bit  */
	int  dlloffset   =    0;            /*offset to divisor latch reg. lsb   */
	int  dlmoffset   =    1;            /*offset to divisor latch reg. msb   */
	int  dll         = 6 ;      /*value for baudrate divisor  lsb    */
	int  dlm         = 0 ;      /*value for baudrate divisor  msb    */

	int  icr;      /*value for control register         */
	int  portadr;      /*value for port address             */


  if (baudrate >= 19200)
  {
	portadr= cpvars[cport].combase;
	icr= inportb(cpvars[cport].lcr);        /*save original control byte */
	delay(1);
	outportb(cpvars[cport].lcr,setdlab);  /*set access to divisor latch*/
	delay(1);
    outportb(portadr + dlloffset, dll);       /*send lsb divisor value     */
    delay(1);
    outportb(portadr + dlmoffset,dlm);       /*send msb divisor value     */
	delay(1);
	outportb(cpvars[cport].lcr,icr);       /*restore control byte       */
  }
}
/*--------------------------------------------------------------*/


int checkhandshake(int cport)
{
	int result = 1;

	switch (cpvars[cport].handshakemode)
	{
		case HND_NONE: result = 1; break;
		case HND_HARD: result =
			(inportb(cpvars[cport].msr) &
			cpvars[cport].hardhandshake) == cpvars[cport].hardhandshake;
		break;
		case HND_SOFT: result = cpvars[cport].xhsflag;
		break;
	}
	return(result);
}
/*--------------------------------------------------------------*/

void sendcom(int cport,
		 char c,       /*character to send out com port*/
		 int *errcode)   /*error code for sendcom       */
										 /*  0 = no error                */
										 /*  8 = time out error          */
										 /* 10 = com port not initialized*/
/*this function sends a character out the com port.                     */
{
int timeout = 5000;
int counter;


	if (!(cpvars[cport].comflag)) /*make sure com port has been initialized*/
		*errcode = ERR_INIT;
	else
	{
		outportb(cpvars[cport].mcr, 0xb);     /* turn on out2, dtr and rts   */
		counter = 0;        /* initialize time out counter          */
		while ((counter < timeout) &&                        /*check for time out*/
			(!(checkhandshake(cport)) || ((inportb(cpvars[cport].lsr) & 0x20) != 0x20)))
		{
			counter++;
			delay(1);          /*delay 1 millisecond - causes timeout at 5 seconds*/
		}
		if (counter ==  timeout)
			*errcode = ERR_TIME;
		else
		{
			disable();          /*no interrpts*/
			outportb(cpvars[cport].thr,c);  /*transmit character*/
			enable();          /*interrupts on*/
			*errcode = ERR_NONE;
		}
	}
}
/*--------------------------------------------------------------*/


#ifdef __cplusplus
void interrupt comisr0(...)
#else
void interrupt far comisr0(void)
#endif
{
	if (cpvars[0].comflag)
	{
		/*get character - store in circular buffer*/
		cpvars[0].curchar = inportb(cpvars[0].rbr);
		if (cpvars[0].handshakemode == HND_SOFT)
		{
			if (cpvars[0].curchar == cpvars[0].xoffchar)
			{
				cpvars[0].xhsflag = 0;
				cpvars[0].putinbuffer = 0;
			}
			else if (cpvars[0].curchar == cpvars[0].xonchar)
			{
				cpvars[0].xhsflag = 1;
				cpvars[0].putinbuffer = 0;
			}
			else
				cpvars[0].putinbuffer = 1;
		}
		else
			cpvars[0].putinbuffer = 1;
		if (cpvars[0].putinbuffer)
		{
			putpntrelement(cpvars[0].combuffer,cpvars[0].bufferin, cpvars[0].curchar);
			/*increment bufferin pointer*/
			cpvars[0].bufferin+= 1;
			/*wrap buffer pointer around to start if > maxbuffer*/
			if (cpvars[0].bufferin > cpvars[0].maxbuffer)
			cpvars[0].bufferin = 0;
			/*current number of characters in buffer incremented 1*/
			cpvars[0].bufferlength+= 1;
			/*disable rts if bufferlength exceeds nearfull constant*/
			if (cpvars[0].bufferlength > cpvars[0].nearfull)
			{
				switch (cpvars[0].handshakemode)
				{
					case HND_NONE: break ;
					case HND_HARD: outportb(cpvars[0].mcr, 9);
					break; /*disable rts , leave dtr and out2 set*/
					case HND_SOFT: outportb(cpvars[0].thr, cpvars[0].xoffchar);
					break; /*transmit character*/
				}
				cpvars[0].hardhsflag = 1;           /*buffer full handshake = true*/
			}
		}
	}
	outportb(0x20, 0x20);
}

/*--------------------------------------------------------------*/

#ifdef __cplusplus
void interrupt comisr1(...)
#else
void interrupt far comisr1(void)
#endif
{
	if (cpvars[1].comflag)
	{
		/*get character - store in circular buffer*/
		cpvars[1].curchar = inportb(cpvars[1].rbr);
		if (cpvars[1].handshakemode == HND_SOFT)
		{
			if (cpvars[1].curchar == cpvars[1].xoffchar)
			{
					cpvars[1].xhsflag = 0;
					cpvars[1].putinbuffer = 0;
			}
			else if (cpvars[1].curchar == cpvars[1].xonchar)
			{
				cpvars[1].xhsflag = 1;
				cpvars[1].putinbuffer = 0;
			}
			else
				cpvars[1].putinbuffer = 1;
		}
		else
			cpvars[1].putinbuffer = 1;
		if (cpvars[1].putinbuffer)
		{
			putpntrelement(cpvars[1].combuffer,cpvars[1].bufferin,cpvars[1].curchar);
			/*increment bufferin pointer*/
			cpvars[1].bufferin+= 1;
			/*wrap buffer pointer around to start if > maxbuffer*/
			if (cpvars[1].bufferin > cpvars[1].maxbuffer)
				cpvars[1].bufferin = 0;
			/*current number of characters in buffer incremented 1*/
			cpvars[1].bufferlength+= 1;
			/*disable rts if bufferlength exceeds nearfull constant*/
			if (cpvars[1].bufferlength > cpvars[1].nearfull)
			{
				switch (cpvars[1].handshakemode)
				{
					case HND_NONE: break;
					case HND_HARD: outportb(cpvars[1].mcr,9);
					break;  /*disable rts , leave dtr and out2 set*/
					case HND_SOFT: outportb(cpvars[1].thr,cpvars[1].xoffchar);
					break; /*transmit character*/
				}
				cpvars[1].hardhsflag = 1;           /*buffer full handshake = true*/
			}
		}
	}
	outportb(0x20,0x20);
}
/*--------------------------------------------------------------*/

/*this functions initializes the com port number, baud rate, parity, stop  */
/*bits and word length.                                                    */
void opencom(int comport,      	/*com port # - 0 or 1          */
		 int  baud,      		/*baud rate - 110,150,300..9600*/
		 int parity,         	/*parity 0 = no parity       */
								/*       1 = even parity     */
								/*       3 = odd parity      */
		 int   stopbits,      	/*stop bits - 1 or 2           */
		 int   numbits,      	/*word length - 7 or 8         */
		 int *errcode)     		/* error code for opencom     */
											/*   0 = no errors             */
											/*   1 = com port error        */
											/*   2 = bauderror            */
											/*   3 = parity error          */
											/*   4 = stop bits error       */
											/*   5 = word length error     */

{
	int comdata, intlevel;
	unsigned short temp;
	*errcode = ERR_NONE;

	comdata = 0;
	if ((numbits == 7) || (numbits == 8))
		comdata = comdata | (numbits-5) ;
	else
		*errcode = ERR_WORD;

	if ((stopbits == 2) || (stopbits == 1))
		comdata = comdata | ((stopbits-1) << 2) ;
	else
		*errcode = ERR_STOP;

	if ((parity == 1) || (parity == 3) || (parity == 0))
		comdata = comdata | (parity << 3) ;
	else
		*errcode = ERR_PARITY;

	switch (baud)
	{
		case 110: comdata = comdata | 0x00;	break;
		case 150: comdata = comdata | 0x20;	break;
		case 300: comdata = comdata | 0x40;	break;
		case 600: comdata = comdata | 0x60;	break;
		case 1200: comdata = comdata | 0x80; break;
		case 2400: comdata = comdata | 0xa0; break;
		case 4800: comdata = comdata | 0xc0; break;
		case 9600 : comdata = comdata | 0xe0; break;
		case 19200: comdata = comdata | 0xe0; break;
		default : *errcode = ERR_BAUD; break;
	}
	if ((comport < COM_1) || (comport > COM_2))
		*errcode = ERR_COM;
	if (*errcode == ERR_NONE)
	{
		bioscom (0, comdata, comport);
		cpvars[comport].thr = cpvars[comport].combase;
		cpvars[comport].rbr = cpvars[comport].combase;
		cpvars[comport].ier = cpvars[comport].combase+1;
		cpvars[comport].lcr = cpvars[comport].combase+3;
		cpvars[comport].mcr = cpvars[comport].combase+4;
		cpvars[comport].lsr = cpvars[comport].combase+5;
		cpvars[comport].msr = cpvars[comport].combase+6;
		intlevel = 8 + cpvars[comport].comintr;
		cpvars[comport].oldfunc = getvect(intlevel);
		if (comport == COM_1)
		   setvect(intlevel,comisr0);
		if (comport == COM_2)
			setvect(intlevel,comisr1);
		disable();                     /*no interrupts*/
		temp = inportb(cpvars[comport].lcr) & 0x7f;  /*turn off divisor access latch*/
		outportb(cpvars[comport].lcr, temp);
		temp = inportb(cpvars[comport].lsr);        /*reset any pending errors*/
		temp = inportb(cpvars[comport].rbr);        /*read any pending character*/
		cpvars[comport].imr = inportb(0x21);
		temp =  0xFF & ~( 1 << cpvars[comport].comintr);
		temp = cpvars[comport].imr & temp;
		outportb(0x21,temp) ;
		outportb(cpvars[comport].ier, 1);      /*enable data ready interrupt*/
		temp = inportb(cpvars[comport].mcr) | 0x08;
		outportb(cpvars[comport].mcr, temp);
		outportb(cpvars[comport].mcr, 0xb);

		if (baud >= 19200)
			setserial(comport,19200);
		enable();                          /*turn on interrupts*/
		cpvars[comport].combuffer = (char *) malloc(cpvars[comport].maxbuffer+1);
		cpvars[comport].comflag = 1;      /*com inititalization flag true*/
		cpvars[comport].hardhsflag = 0;   /*buffer full handshake false*/
		cpvars[comport].xhsflag = 1;
		cpvars[comport].bufferin = 0;     /*set circular buffer input to 0*/
		cpvars[comport].bufferout = 0;    /*set circular buffer output to 0*/
		cpvars[comport].bufferlength = 0; /*set bufferlength to 0*/
	}
}
/*--------------------------------------------------------------*/


/*this function disables the com port interrupt.*/
void closecom(int cport)
{
	int temp;
	if (cpvars[cport].comflag)
	{
		disable();
		temp = inportb(0x21);
		temp = temp | (cpvars[cport].imr  & 0x18);
		outportb(0x21,cpvars[cport].imr);
		temp = inportb (cpvars[cport].lcr) & 0x7f;
		outportb (cpvars[cport].lcr, temp);
		outportb (cpvars[cport].ier,  0);
		outportb (cpvars[cport].mcr,  0);
		temp = cpvars[cport].comintr + 8;
		setvect (temp,cpvars[cport].oldfunc);
		enable();
		free (cpvars[cport].combuffer);
		cpvars[cport].comflag = 0;
	}
}

/*--------------------------------------------------------------*/

/*this function returns 1 character from the combuffer, at the array element*/
/*pointed to by the circular buffer pointer bufferout.                       */

void checkcom(int cport,
		  char *c,      /*returns character from combuffer*/
		  int *errcode)  /*error code for checkcom       */
										   /*   0 = no error                */
										   /*   6 = no character available  */
										   /*   7 = buffer overflow         */

{
	if (cpvars[cport].bufferin == cpvars[cport].bufferout)
		*errcode = ERR_CHAR;
	else
	{
		if (cpvars[cport].bufferlength > cpvars[cport].maxbuffer)
			*errcode = ERR_BUFF;
		else
		{
			*c =  getpntrelement(cpvars[cport].combuffer, cpvars[cport].bufferout);   /*get charater out of buffer*/
			cpvars[cport].bufferout+= 1;       /*increment bufferoutpointer*/
			/*wrap bufferout pointer around if >  maxbuffer*/
			if (cpvars[cport].bufferout > cpvars[cport].maxbuffer)
				cpvars[cport].bufferout = 0;
			cpvars[cport].bufferlength -= 1;
			/*enable rts if bufferlength < nearempty*/
			if (cpvars[cport].hardhsflag  &&
				(cpvars[cport].bufferlength < cpvars[cport].nearempty))
			{
				switch (cpvars[cport].handshakemode)
				{
					case HND_NONE: break;
					case HND_HARD: outportb(cpvars[cport].mcr, 0xb); break;
					case HND_SOFT: sendcom(cport,cpvars[cport].xonchar,&cpvars[cport].xerr);break;
				}
				cpvars[cport].hardhsflag = 0;
			}
		}
		*errcode = ERR_NONE;                      /*good character read*/
	}
}

/*--------------------------------------------------------------*/

void writecom(int cport,
		char *s,     /*string to send out com port*/
		int *errcode) /*error code for writelncom*/
                                         /*  0 = no error                */
                                         /*  8 = time out error          */
                                         /* 10 = com port not initialized*/

{
	int  i,l;

	i = 0;
	*errcode = ERR_NONE;
	l = strlen(s);
	for (i = 0; i < l; i++)
		sendcom(cport, s[i], errcode);            /*send character out com port */
}
/*--------------------------------------------------------------*/

void writelncom(int cport,
		char *s,     /*string to send out com port*/
		int *errcode) /*error code for writelncom*/
										 /*  0 = no error                */
										 /*  8 = time out error          */
										 /* 10 = com port not initialized*/


{
	writecom(cport,s,errcode);
	sendcom(cport,13,errcode);
}
/*--------------------------------------------------------------*/

/*this function inputs a string variable until combuffer is empty or a      */
/* delimiter (carriage return in this case) is received.                      */
/*if over 255 characters are input without a delimiter error #9 results.      */

void readlncom(int cport,
		char *s,       /*returns string from combuffer */
		int *errcode) /*readlncom error code          */
											 /* 0 = no error                */
											 /* 6 = no character available  */
											 /* 7 = buffer overflow         */
											 /* 9 = string variable overrun */

{
	char  dl = 13; /*string delimiter - current value for cr delimiter     */
	int i = 0;
	char c;
	long counter;

	if (cpvars[cport].bufferlength > cpvars[cport].maxbuffer)
		*errcode = ERR_BUFF;
	else
	{
		counter = 0;
		do
		{
			checkcom(cport, &c, errcode);             /*get character*/
			if (*errcode == ERR_NONE)
			{
				if (c != 13)
				{
					s[i] = c; /*if character present add */
					i++;
				}
			}
			else
			{
				delay(1);
				counter++;
			}
		/*repeat until buffer *empty or cr or max string length reached*/
		} while ((counter < timeout) && (c != dl) && (i < 255));
		if (i == 255)
			*errcode = ERR_MAXSTR;
		else
			*errcode = ERR_NONE;
		s[i] = 0;
	}
}
/*--------------------------------------------------------------*/


#ifdef __cplusplus
}
#endif
