/***************************************************************************
	(C) COPYRIGHT 1999,2000,2001 by JANOS TECHNOLOGY INC. Townshend Vermont
	PROPRIETARY: Source code, algorithms and data contained in this
				 file may not be used, reproduced or transmitted
				 without written permission of JANOS TECHNOLOGY INC.

		 PROJECT: AGM300
		  AUTHOR: Chris Prozzo

***************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <stdio.h>
#include <dos.h>

#include "culproto.h"
#include "pc2hgm.h"

#define MAIN_HELP_ROW 2
#define MAIN_HELP_COL 1
#define PULLDN_HELP_ROW 24
#define PULLDN_HELP_COL 1

/*.............................LOCAL FUNCTIONS.............................*/
void edit_system_parameters(void);
void edit_zone_parameters(void);
void set_structure_colors(void);
void edit_cal(void);
void put_fault_log(void);
void show_sensors(void);
void log_sensors(void);


void quit(void);
void about(void);
void path_init(void);
void choose_config(void);
void save_config(void);
/*...........................EXTERNAL FUNCTIONS............................*/

void dir();
void set_clock();

/*.......................PULL DOWN SUB-MENU SELECTIONS.....................*/

char *file_menu[] ={"Open Setup","Save Setup","Path","Directory","About"};
char *edit_menu[] ={"System","Zones","Calibration"};
char *diagnostic_menu[] ={"Faults","Sensors","Enter Service Mode","Release Service Mode"};
char *HGM_menu[] ={"Get Setup","Send Setup","Zone Hold","Release Hold","Get Trend data","Set AGM clock","Ack Alarm","Alarm Log"};
char *quit_menu[]={"Exit Program"};

/*...........................SUB-MENU FUNCTIONS...........................*/

void (*file_funcs[])() = { choose_config,save_config,set_path,dir,about};
void (*edit_funcs[])() = { edit_system_parameters,edit_zone_parameters,edit_cal};
void (*diagnostic_funcs[])() = { put_fault_log,show_sensors,request_service_mode,release_service_mode};
void (*HGM_funcs[])() = {get_HGM_parameters,send_HGM_parameters,zone_hold,release_hold,get_trend,set_HGM_clock,ack_alarm,put_alarm_log};
void (*quit_funcs[])() = { quit };

/*.......................MAIN MENU HELP MESSAGES..........................*/

ESI_MSG main_help[] =
{
	{"Store, retrieve and erase data files on disk",WHITE,BLACK,MAIN_HELP_ROW,MAIN_HELP_COL},
	{"View/Edit System Parameters",WHITE,BLACK,MAIN_HELP_ROW,MAIN_HELP_COL},
	{"Read/Write Setup Data to AGM",WHITE,BLACK,MAIN_HELP_ROW,MAIN_HELP_COL},
	{"View Faults or Sensor data",WHITE,BLACK,MAIN_HELP_ROW,MAIN_HELP_COL},
	{"Exit AGM interface",WHITE,BLACK,MAIN_HELP_ROW,MAIN_HELP_COL},
};

/*......................PULL DOWN MENU HELP ARRAYS.........................*/

ESI_MSG file_help[] =
{
	{"Get setup parameters from disk at the current path",WHITE,BLACK,PULLDN_HELP_ROW,PULLDN_HELP_COL},
	{"Save setup parameters to disk at current path",WHITE,BLACK,PULLDN_HELP_ROW,PULLDN_HELP_COL},
	{"Set the drive and subdirectory to use for disk operations",WHITE,BLACK,PULLDN_HELP_ROW,PULLDN_HELP_COL},
	{"View the contents of disk at the current path",WHITE,BLACK,PULLDN_HELP_ROW,PULLDN_HELP_COL},
	{"Information about the program",WHITE,BLACK,PULLDN_HELP_ROW,PULLDN_HELP_COL},
};

ESI_MSG edit_help[] =
{
	{"Edit system parameters",WHITE,BLACK,PULLDN_HELP_ROW,PULLDN_HELP_COL},
	{"Edit Zone parameters",WHITE,BLACK,PULLDN_HELP_ROW,PULLDN_HELP_COL},
	{"Edit Cal or add new gas",WHITE,BLACK,PULLDN_HELP_ROW,PULLDN_HELP_COL},
};

ESI_MSG diagnostic_help[] =
{
	{"View fault log",WHITE,BLACK,PULLDN_HELP_ROW,PULLDN_HELP_COL},
	{"View raw sensor data",WHITE,BLACK,PULLDN_HELP_ROW,PULLDN_HELP_COL},
	{"Put AGM in SERVICE MODE (Alarms defeated, Measurements Stopped",WHITE,BLACK,PULLDN_HELP_ROW,PULLDN_HELP_COL},
	{"Return to normal measurement mode",WHITE,BLACK,PULLDN_HELP_ROW,PULLDN_HELP_COL},
};

ESI_MSG HGM_help[] =
{
	{"Read All Setup Data in AGM ",WHITE,BLACK,PULLDN_HELP_ROW,PULLDN_HELP_COL},
	{"Write Current Setup Data to AGM ",WHITE,BLACK,PULLDN_HELP_ROW,PULLDN_HELP_COL},
	{"Jump to Zone and Hold ",WHITE,BLACK,PULLDN_HELP_ROW,PULLDN_HELP_COL},
	{"Release Zone Hold Mode ",WHITE,BLACK,PULLDN_HELP_ROW,PULLDN_HELP_COL},
	{"Read Trend Log for a Zone ",WHITE,BLACK,PULLDN_HELP_ROW,PULLDN_HELP_COL},
	{"Set AGM Time and Date to PC Clock ",WHITE,BLACK,PULLDN_HELP_ROW,PULLDN_HELP_COL},
	{"Acknowlege Alarm on a single zone ",WHITE,BLACK,PULLDN_HELP_ROW,PULLDN_HELP_COL},
	{"View Alarm Log",WHITE,BLACK,PULLDN_HELP_ROW,PULLDN_HELP_COL},

};

ESI_MSG quit_help[] =
{
	{"Unsaved Data Will Be Lost!",WHITE,BLACK,PULLDN_HELP_ROW,PULLDN_HELP_COL},
};

/*...........................MAIN MENU STRUCTURES.........................*/

POP_MENU main_menu_sel[] =
{
	{ " File ",file_menu,5,5,WHITE,BLACK,-1,file_funcs,file_help},
	{ " Edit ",edit_menu,3,3,WHITE,BLACK,-1,edit_funcs,edit_help},
	{ " AGM ",HGM_menu,8,8,WHITE,BLACK,-1,HGM_funcs,HGM_help},
	{ " Diagnostics",diagnostic_menu,4,4,WHITE,BLACK,-1,diagnostic_funcs,diagnostic_help},
	{ " Quit ",quit_menu,1,1,WHITE,BLACK,-1,quit_funcs,quit_help},

};

/*........................SET MENU STRUCTURE COLORS.......................*/
/* Initializes pull down menu colors                                      */

void set_menu_colors()
{
	int i;

	for(i=0;i<=7;i++)
	{
		HGM_help[i].forg = help_fg;
		HGM_help[i].back = help_bk;
		if(i<5)
		{
			file_help[i].forg = help_fg;
			file_help[i].back = help_bk;
			main_menu_sel[i].forg = pop_menu_fg;
			main_menu_sel[i].back = pop_menu_bk;
			main_help[i].forg = help_fg;
			main_help[i].back = help_bk;
		}
		if(i<4)
		{
			diagnostic_help[i].forg = help_fg;
			diagnostic_help[i].back = help_bk;
		}
		if(i<3)
		{
			edit_help[i].forg = help_fg;
			edit_help[i].back = help_bk;

		}
		if(i<2)
		{



		}
		if(i<1)
		{
			quit_help[i].forg = help_fg;
			quit_help[i].back = help_bk;
		}
	}

	set_shadow(1,WHITE,BLACK,0);
	return;
}

/*................................MAIN MENU...............................*/

void main_menu()
{

	delay(0); /*calibrate delay clock*/
	mouse_on();
	curlocat(0,0);
	colrprts("                   BACHARACH INCORPORATED - AGM PC INTERFACE               ",title_fg,title_bk);
	do
	{
		linemenu("",0,1,main_menu_fg,main_menu_bk,highlight_fg,highlight_bk,main_menu_sel,5,0,1,main_help,&main_menu_return_val);
	}while(1);
}

/*................................QUIT....................................*/

void quit()
{
	int return_val;
	char tab[2] = {9,0};
	char *msg[] = {"       EXITING PROGRAM",
				   " UNSAVED DATA WILL BE LOST !"};
	delay(400);
	clearkbd();
	kybdstuf(tab);
	return_val = dlg_answ(1,4,24,-1,msg_fg,msg_bk,border_fg,border_bk,msg,2);
	curoff();
	delay(200);
	if (return_val != 0)
		return;
	diskstat(1);
	mouse_off();
	curon();
	clrscr();
	exit(0);
}

/*................................ABOUT...................................*/
/* Put up information box for a few seconds                               */

void about()
{
	DLG_PTR dlg_ptr;
	int f,b;

	f=pop_menu_fg;
	b=pop_menu_bk;
	dlg_ptr = dlg_open(4,15,16,65,-2,f,b,f,b);
	if(dlg_ptr != NULL)
	{
		dlg_loc(dlg_ptr,0,17);
		dlg_prnt(dlg_ptr,"AGM PC INTERFACE",f,b,0);
		dlg_loc(dlg_ptr,1,17);
		dlg_prnt(dlg_ptr,"- Version 1.52 -",f,b,0);
		dlg_loc(dlg_ptr,2,11);
		dlg_prnt(dlg_ptr,"for AGM300 1.2X-1.5X firmware",f,b,0);
		dlg_loc(dlg_ptr,3,14);
		dlg_prnt(dlg_ptr,"(c)2003 BACHARACH INC",f,b,0);

		dlg_loc(dlg_ptr,10,2);
		dlg_prnt(dlg_ptr,"          -HIT ANY KEY TO CONTINUE-",f,b,0);
		syspsany(0,1,0,0);
		dlg_clos(dlg_ptr);
	}

	clearkbd();
	kybdstuf("f");
	return;
}



