/*
	REV 0.1     fixed node crc prob
	REV 0.2 	added alarm log, service mode, com retry
	REV 0.3     added new vars to diagnostic screen and zone temp entry
	REV 1.0     first release
	REV 1.1     modbus fix, new ack mode
	REV 1.12	Added stop bit selection, corrected CRC byte order
	REV 1.13    Even # of bytes to match HGM 1.13
	REV 1.14    corrected cal struct back to 174bytes
	REV 1.15    added R408A in place of future1
	REV 1.20    Sync to 1.2x
	REV 1.21    6/14/2001 convert all PPMs from float to Uint before
				display to fix rounding error. System, zone and cal data
				are now sent back to HGM automatically after an edit. Alarm
				ack is now a silence.
	REV 1.22    7/23/2001 corrected gas names, allow all zone edits before sending.
	REV 1.23    7/24/2001 added "download now?" dialog to edits
	REV 1.40    10/xx/2001 sync'd ref to hgm firmware 1.4x
	REV 1.41    1/30/2002 changed order of 'send data' to put zone data last
				due to long EE write time
	REV 1.42    2/15/2002 fixed missing day1 digit is set clock function
	REV 1.50 	7/11/2003 added send single zone data for alarm ack timeout
	REV 1.51	10/8/2003 changed timeout tries from 4000 to 10000 on get_ser_chars
	REV 1.52    05/27/2005 moved clear buffer to before send query, added
				1 sec delay after each receive in get_data

*/

/*...........................GLOBAL CONSTANTS.............................*/


#define TRUE 1
#define FALSE 0
#define ON 1
#define OFF 0
#define YES 1
#define OK 1
#define NO 0
#define SUCCESS 1
#define FAILED 0
#define METRIC 0
#define ENGLISH 1


/*...IO CONSTANTS...*/

#define NO_HANDSHAKE 0
#define HARD_HANDSHAKE 1
#define SOFT_HANDSHAKE 2
#define CTS_DTR 0x30
#define CTS 0x10
#define DTR 0x20
#define XON_DEF 0x17
#define XOFF_DEF 0x19


/* OUTPUT DEVICE CONSTANTS */

#define PORTRAIT 0
#define LANDSCAPE 1
#define GRAPHIC_PRINTER 0
#define DATA_PRINTER 1
#define NUM_OUTPUT_DEVICES 2



//.........................................................................

#include "ae.h"
#include "hgmtypes.h"  // contains typdefs for hgm300 data types

#if defined (MAIN) // controls use of "extern" for var declarations
// Global variable declaration and definitions...............................
// structures defined above are used and assigned default values

//..........................................................................
//..........................................................................

HGM_CAL_TYPE HGM_cal={
	{1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,  // default all gases to nominal curve
	 1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,
	 1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,
	 1.0,1.0,1.0},
	 {R134A,R134A,R134A},
	 {"NEW GAS1","NEW GAS2","NEW GAS3"}}; // default new gases to R134a cal
//..........................................................................
HGM_SYSTEM_DATA HGM=  // default values
{
	300, // type detect uninitialized eeprom, set to 300 if initialized
	0.0, // firmware revision level
	0,   // unit serial number
	1,   // network slave node number 1-7 only
	"HGM300 TAG  ", // location tag 12chars max
	1,   // stop bits for RS485
	AP_UNUSED, // audible alarm parameter
	0,   // alarm ack mode
	4,   // number of installed zone valves
	{0,0,0,0,0,0,0,0,0,0,0,0,0}, // Time and date of last gas cal
	AUTO_ZERO, // what triggers a rezero
	15, // number of minutes before releasing a zone hold
	0, // report ppm/l unless >= this value
	16, // size of running average
	0.016,// CH2 loop2 factor ma/ppm
	60, // minutes to service mode timeout
	9, // baud rate for RS485
	{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0} // production test flags
};
//..........................................................................
HGM_STATUS_DATA HGM_status={NORMAL_MODE,IDLE,NO,0,0,0,0,NO,{{0}}};
//..........................................................................
HGM_SENSOR_DATA sensor={0.0,0.0,0.0,0.0,0.0,
						0.0,0.0,0.0,0.0,0.0,
						0,0.0,0.0,0.0,0.0,
						0.0,0.0,0.0,0.0,0.0,
						0.0};
//..........................................................................
HGM_SENSOR_DATA unit[4]; // for cal data logging of 4 units
//..........................................................................
HGM_FAULT_LOG_TYPE HGM_fault_log;
HGM_ALARM_LOG_TYPE HGM_alarm_log;
HGM_ZONE_TYPE zone[16];
HGM_LOG_TYPE zone_log;
HGM_TIME_DATA HGM_time;

char *refrigerant_names[MAX_GASES]=
{"R11","R12","R22","R23","R113",
 "R114","R123","R124","NH3","R401A",
 "R402A","R402B","R404A","R407A","R407C",
 "R409A","R410A","R500","R502","R503",
 "R507","R508B","H1301","R408A","FUTURE2",
 "FUTURE3","FUTURE4","FUTURE5","FUTURE6","FUTURE7",
 "NEW GAS1","NEW GAS2","NEW GAS3"};

#else
// global vars extern........................................................
extern HGM_CAL_TYPE HGM_cal;
extern HGM_SYSTEM_DATA HGM;
extern HGM_STATUS_DATA HGM_status;
extern HGM_SENSOR_DATA sensor;
extern HGM_SENSOR_DATA unit[4]; // for cal data logging of 4 units
extern HGM_FAULT_LOG_TYPE HGM_fault_log;
extern HGM_ALARM_LOG_TYPE HGM_alarm_log;
extern HGM_ZONE_TYPE zone[16];
extern HGM_LOG_TYPE zone_log;
extern HGM_TIME_DATA HGM_time;
extern refrigerant_names[30][9];
//.........................................................................
#endif

/*...........................TYPE DEFINTIONS..............................*/




/*...........................GLOBAL VARIABLES.............................*/
#if defined (MAIN)

/* STATUS FLAGS AND VARIABLES */

	int main_menu_return_val;	/* value returned by pull down menu function */
	int mouse,esimouse;			/* true if microsoft compatable mouse detected */
	int help;					/* flag for context sensitive help */
	int sound_status; 			/* when ON speaker beeps */
	int in_service_mode=0;       /* permits special functions */
char trend_string[100][30];
char *trend_string_ptr[100];

/* GRAPHICS AND PLOTTING PARAMETERS */

	int axes_type;
	int grids;
	double x_axes_max;
	double x_axes_min;
	double t_axes_max;
	double t_axes_min;
	double x_intercept;
	double y_intercept;
	double od_axes_max;
	double od_axes_min;
	double od_tics;
	double x_tics;
	double t_tics;
	char xaxis_label[16];
	char yaxis_label[16];




/* TEXT SCREEN COLORS  _fg = foreground  _bk = backround */

	int title_fg,
		title_bk;
	int msg_fg,
		msg_bk;
	int main_menu_fg,
		main_menu_bk;
	int highlight_fg,
		highlight_bk;
	int pop_menu_fg= 0,
		pop_menu_bk=7;
	int frame_fg,
		frame_bk;
	int frame_title_fg,
		frame_title_bk;
	int dlgbox_fg,
		dlgbox_bk;
	int border_fg,
		border_bk;
	int help_fg,
		help_bk;
	int text_fg,
		text_bk;
	int user_input_fg,
		user_input_bk;






/* MISC */
	char cur_date_str[30];
	char cur_time_str[30];
	char date_string[12];


/*------------------------------------------------------------------------*/

#else

/* STATUS FLAGS AND VARIABLES */

	extern float version;
	extern int main_menu_return_val;
	extern int mouse,esimouse;
	extern int help;
	extern int units;
	extern int sound_status;
	extern int in_service_mode;

extern char trend_string[100][30];
extern char *trend_string_ptr[100];

/* GRAPHICS AND PLOTTING PARAMETERS */

	extern int axes_type;
	extern int grids;
	extern double x_axes_max;
	extern double x_axes_min;
	extern double t_axes_max;
	extern double t_axes_min;
	extern double x_intercept;
	extern double y_intercept;
	extern double od_axes_max;
	extern double od_axes_min;
	extern double od_tics;
	extern double x_tics;
	extern double t_tics;
	extern char xaxis_label[16];
	extern char yaxis_label[16];



/* TEXT SCREEN COLORS */

	extern int 	title_fg,
				title_bk;
	extern int 	msg_fg,
				msg_bk;
	extern int  main_menu_fg,
				main_menu_bk;
	extern int 	highlight_fg,
				highlight_bk;
	extern int 	pop_menu_fg,
				pop_menu_bk;
	extern int 	dlgbox_fg,
				dlgbox_bk;
	extern int 	border_fg,
				border_bk;
	extern int	help_fg,
				help_bk;
	extern int 	text_fg,
				text_bk;
	extern int 	user_input_fg,
				user_input_bk;




/* MISC */

	extern char cur_date_str[30];
	extern char cur_time_str[30];
	extern char date_string[12];
#endif



/*.............GLOBAL FUNCTIONS  (Listed by source file)...................*/

/* MAIN.C */




/* COM_IO.C */

int	open_com(int port);
int com_error(int error_code);
int get_RTU_response(int port,                // com port to use
					 unsigned char slave,     // slave node
					 unsigned char function,  // modbus funtion type
					 unsigned int REG_address,// Register address
					 unsigned char *data,     // source or destination of data
					 unsigned int size);       // number of bytes to transfer

int send_RTU_query(int port,                // com port to use
				   unsigned char hgm,       // HGM #
				   unsigned char function,  // modbus funtion type
				   unsigned int REG_address,// Register address
				   unsigned char *data,     // source or destination of data
				   unsigned int size);       // number of bytes to transfer

int get_HGM_parameters(void);
int send_HGM_parameters(void);
int get_fault_log(void);
int get_sensor_data(void);
int get_trend_data(unsigned int zone);
int get_cal_data(void);
int send_zone_data(void);
int send_single_zone_data(int z);
int send_system_data(void);
int send_cal_data(void);
int send_zone_hold(void);
int send_release_hold(void);
int send_service_mode_request(void);
int send_service_mode_release(void);


/* MENU.C */

void main_menu();



/* USER_IO.C */
void put_system_box(void);
void put_zone_box(void);
void put_system_data(void);
void put_status_data(void);
void put_zone_data(void);
void release_hold(void);
void zone_hold(void);
void request_service_mode(void);
void release_service_mode(void);
void get_trend(void);
void set_HGM_clock();
void ack_alarm(void);
void put_fault_log(void);
void put_alarm_log(void);

/* FILE_IO.C */

void path_init();
void save_config();
void get_config();
void choose_config();
void save_trend();
void set_path();
void save_job();
void get_job();
void out_of_memory();



/* MEMORY.C */

int data_loss_warning();


/* UTILS.C */

void error_beep();
void good_beep();
void mouse_on();
void mouse_off();
int put_message(DLG_PTR *box_ptr,char *message,int row,int col);


